Attribute VB_Name = "modDialog"
Option Explicit

'******************************
' API-Deklarationen fr Dialoge.
'******************************

' Diese Funktion ldt den Dialog aus der Ressourcendatei und erstellt ber
' 'CreateWindowEx' die Fenster, die in der Dialogressource angegeben sind. Der
' Parameter 'lpDialogFunc' ist ein Zeiger auf die Rckrufprozedur des Dialogs.
Public Declare Function CreateDialogParam Lib "user32.dll" Alias "CreateDialogParamA" (ByVal hInstance As Long, ByVal lpName As String, ByVal hWndParent As Long, ByVal lpDialogFunc As Long, ByVal lParamInit As Long) As Long

' 'EndDialog' entldt den Dialog. Es ist wichtig, dass die Zugriffsnummer des
' Dialogs gespeichert wird, damit der Dialog auch wieder sicher entladen werden kann.
Private Declare Function EndDialog Lib "user32.dll" (ByVal hDlg As Long, ByVal nResult As Long) As Long

'******************************
' Steuerelement-IDs.
'******************************

' Vordefinierte Steuerelement-IDs.
Private Const IDCANCEL As Long = 2&
Private Const IDOK As Long = 1&

'
' Copy of defines in the recource include file.
'
' #define IDC_SHOWMSG                     1000
' #define IDC_BTNENABLED                  1001
' #define IDC_EDITCONTENT                 1002
' #define IDC_MSGTEXT                     1003
' #define IDC_ISENABLED                   1004
' #define IDC_TITLE                       1005
'
Private Const IDC_SHOWMSG As Long = 1000&
Private Const IDC_BTNENABLED As Long = 1001&
Private Const IDC_EDITCONTENT As Long = 1002&
Public Const IDC_MSGTEXT As Long = 1003&    ' Public.
Private Const IDC_ISENABLED As Long = 1004&
Public Const IDC_TITLE As Long = 1005&      ' Public.

' 'GetDlgItem' returns the handle of the control on the dialog specified by
' 'nIDDlgItem'. 'nIDDlgItem' is the ID of the control.
Public Declare Function GetDlgItem Lib "user32.dll" (ByVal hDlg As Long, ByVal nIDDlgItem As Long) As Long

Private Declare Function SetDlgItemText Lib "user32.dll" Alias "SetDlgItemTextA" (ByVal hDlg As Long, ByVal nIDDlgItem As Long, ByVal lpString As String) As Long
Private Declare Function GetDlgItemText Lib "user32.dll" Alias "GetDlgItemTextA" (ByVal hDlg As Long, ByVal nIDDlgItem As Long, ByVal lpString As String, ByVal nMaxCount As Long) As Long
Private Declare Function EnableWindow Lib "user32.dll" (ByVal hWnd As Long, ByVal fEnable As Long) As Long
Private Declare Function DeleteObject Lib "gdi32.dll" (ByVal hObject As Long) As Long

Private Const WM_COMMAND As Long = &H111&
Private Const WM_INITDIALOG As Long = &H110&
Private Const WM_CLOSE As Long = &H10&

Public g_hWndEdit As Long   ' Handle of 'IDC_MSGTEXT'. We need this to update
                            ' 'IDC_EDITCONTENT'.
Public g_hWndDialog As Long ' Handle of dialog.
Public g_hFont As Long      ' Handle of font object used in title label.

Private m_Checked As Boolean

'
' Structure of 'DialogProc' taken from Windows Development SDK Documentation.
'
' BOOL CALLBACK DialogProc(
'   HWND hwndDlg,  // handle to dialog box
'   UINT uMsg,     // message
'   WPARAM wParam, // first message parameter
'   LPARAM lParam  // second message parameter
' );
'
Public Function DialogProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim s As String, n As Long
    
    ' Wenn sich der Text in der TextBox ndert, soll dieser im Label angezeigt
    ' werden.
    If uMsg = WM_COMMAND And lParam = g_hWndEdit Then
        s = Space$(256)
        n = GetDlgItemText(g_hWndDialog, IDC_MSGTEXT, s, Len(s))
        s = Left$(s, n)
        Call SetDlgItemText(g_hWndDialog, IDC_EDITCONTENT, s)
    
    ' Wenn der Dialog initialisiert wird, soll FMain deaktiviert werden.
    ElseIf uMsg = WM_INITDIALOG Then
        FMain.Enabled = False
        
        ' Im SDK steht, dass bei Verarbeitung der Nachrichten 'True'
        ' zurckgegeben werden soll. In diesem Beispiel wird dies nicht berall
        ' gemacht, um den Code auf das Wesentliche zu beschrnkten. Auch bei
        ' 'WM_INITDIALOG' soll man 'True' (1) zurckgeben, damit Windows
        ' automatisch den Fokus auf das nchste Steuerelement im Fenster setzt.
        ' Das funktioniert leider nicht, auch das manuelle Setzen des Fokus
        ' ber 'SetFocus' scheitert. Wahrscheinlich liegt dies an der
        ' undurchschaubaren Verwaltung von Tabstops unter VB.
        DialogProc = 1
        Exit Function
    'ElseIf uMsg = WM_CLOSE Then
    '
    '    ' We do not need any code here because all clean up will be done when
    '    ' a 'WM_COMMAND' message is passed by 'ID_CANCEL'.
    End If
    
    ' Statuswechsel bei der CheckBox, Button (de)aktivieren.
    ' Hier der Einfachheit halber etwas unsauber programmiert.
    If wParam = IDC_BTNENABLED Then
        m_Checked = Not m_Checked
        Call SetDlgItemText(g_hWndDialog, IDC_ISENABLED, CStr(m_Checked))
        Call EnableWindow(GetDlgItem(g_hWndDialog, IDC_SHOWMSG), CLng(IIf(m_Checked, 1&, 0&)))
    
    ' Der "ShowMessage"-Button wurde angeklickt, der Inhalt der TextBox wird in
    ' einem Meldungsfeld ausgegeben.
    ElseIf wParam = IDC_SHOWMSG And uMsg = WM_COMMAND Then
        s = Space$(256)
        n = GetDlgItemText(g_hWndDialog, IDC_MSGTEXT, s, Len(s))
        s = Left$(s, n)
        Call MsgBox(IIf(Len(s) > 0, s, "The textbox is empty!"))
    
    ' "Cancel"-Button.
    ElseIf wParam = IDCANCEL And uMsg = WM_COMMAND Then
        Call EndDialog(g_hWndDialog, 0&)
        Call DeleteObject(g_hFont)
        FMain.lblReturn.Caption = "The ""Cancel"" button was pressed!"
        FMain.Enabled = True
    
    ' "OK"-Button.
    ElseIf wParam = IDOK And uMsg = WM_COMMAND Then
        Call EndDialog(g_hWndDialog, 0&)
        Call DeleteObject(g_hFont)
        FMain.lblReturn.Caption = "The ""OK"" button was pressed!"
        FMain.Enabled = True
    End If
End Function
